@@ -9,7 +9,7 @@ package HTML::Parser;
 use strict;
 use vars qw($VERSION @ISA);
 
-$VERSION = '3.46';  # $Date: 2005/10/24 11:17:49 $
+$VERSION = '3.47';  # $Date: 2005/11/22 21:28:42 $
 
 require HTML::Entities;
 
@@ -353,6 +353,22 @@ behaviour is what at least MSIE does.  Enabling this attribute makes
 closing "</plaintext>" tag effective and the parsing process will resume
 after seeing this tag.  This emulates gecko-based browsers.
 
+=item $p->empty_element_tags
+
+=item $p->empty_element_tags( $bool )
+
+By default, empty element tags are not recognized as such and the "/"
+before ">" is just treated like a nomal name character (unless
+C<strict_names> is enabled).  Enabling this attribute make
+C<HTML::Parser> recognize these tags.
+
+Empty element tags look like start tags, but end with the character
+sequence "/>" instead of ">".  When recognized by C<HTML::Parser> they
+cause an artificial end event in addition to the start event.  The
+C<text> for the artificial end event will be empty and the C<tokenpos>
+array will be undefined even though the the token array will have one
+element containg the tag name.
+
 =item $p->marked_sections
 
 =item $p->marked_sections( $bool )
@@ -460,21 +476,18 @@ This option is only available with perl-5.8 or better.
 =item $p->xml_mode( $bool )
 
 Enabling this attribute changes the parser to allow some XML
-constructs such as I<empty element tags> and I<XML processing
-instructions>.  It disables forcing tag and attribute names to lower
-case when they are reported by the C<tagname> and C<attr> argspecs,
-and suppresses special treatment of elements that are parsed as CDATA
-for HTML.
+constructs.  This enables the behaviour controlled by individually by
+the C<case_sensitive>, C<empty_element_tags>, C<strict_names> and
+C<xml_pic> attributes and also suppresses special treatment of
+elements that are parsed as CDATA for HTML.
 
-I<Empty element tags> look like start tags, but end with the character
-sequence "/>".  When recognized by C<HTML::Parser> they cause an
-artificial end event in addition to the start event.  The C<text> for
-the artificial end event will be empty and the C<tokenpos> array will
-be undefined even though the only element in the token array will have
-the correct tag name.
+=item $p->xml_pic
 
-I<XML processing instructions> are terminated by "?>" instead of a
-simple ">" as is the case for HTML.
+=item $p->xml_pic( $bool )
+
+By default, I<processing instructions> are terminated by ">". When
+this attribute is enabled, processing instructions are terminated by
+"?>" instead.
 
 =back
 
@@ -1211,7 +1224,7 @@ be found at C<http://www.sgml.u-net.com/book/sgml-8.htm>.
 
 =head1 COPYRIGHT
 
- Copyright 1996-2004 Gisle Aas. All rights reserved.
+ Copyright 1996-2005 Gisle Aas. All rights reserved.
  Copyright 1999-2000 Michael A. Chase.  All rights reserved.
 
 This library is free software; you can redistribute it and/or
