@@ -1,6 +1,6 @@
 package HTML::Entities;
 
-# $Id: Entities.pm,v 1.26 2003/08/17 23:40:59 gisle Exp $
+# $Id: Entities.pm,v 1.32 2005/10/24 12:08:13 gisle Exp $
 
 =head1 NAME
 
@@ -16,12 +16,13 @@ HTML::Entities - Encode or decode strings with HTML entities
 
 For example, this:
 
- $input = "vis--vis Beyonc's nave papier-mch rsum";
- print encode_entities($in), "\n"
+ $input = "vis--vis Beyonc's nave\npapier-mch rsum";
+ print encode_entities($input), "\n"
 
 Prints this out:
 
- vis-&agrave;-vis Beyonc&eacute;'s na&iuml;ve papier-m&acirc;ch&eacute; r&eacute;sum&eacute;
+ vis-&agrave;-vis Beyonc&eacute;'s na&iuml;ve
+ papier-m&acirc;ch&eacute; r&eacute;sum&eacute;
 
 =head1 DESCRIPTION
 
@@ -30,22 +31,47 @@ character entities.  The module provides the following functions:
 
 =over 4
 
-=item decode_entities($string)
+=item decode_entities( $string, ... )
 
 This routine replaces HTML entities found in the $string with the
-corresponding ISO-8859-1 character, and if possible (under perl 5.8
-or later) will replace to Unicode characters.  Unrecognized
+corresponding Unicode character.  Under perl 5.6 and earlier only
+characters in the Latin-1 range are replaced. Unrecognized
 entities are left alone.
 
+If multiple strings are provided as argument they are each decoded
+separately and the same number of strings are returned.
+
+If called in void context the arguments are decoded in-place.
+
 This routine is exported by default.
 
-=item encode_entities($string, [$unsafe_chars])
+=item _decode_entities( $string, \%entity2char )
+
+=item _decode_entities( $string, \%entity2char, $allow_unterminated )
+
+This will in-place replace HTML entities in $string.  The %entity2char
+hash must be provided.  Named entities not found in the %entity2char
+hash are left alone.  Numeric entities are always expanded.
+
+If $allow_unterminated is TRUE then we also unterminated named
+entities will also be expanded.  The longest matching name in
+%entity2char will be used.
+
+   $string = "foo&nbspbar";
+   _decode_entities($string, { nb => "@", nbsp => "\xA0" }, 1);
+   print $string;  # will print "foobar"
+
+This routine is exported by default.
+
+=item encode_entities( $string )
+
+=item encode_entities( $string, $unsafe_chars )
 
 This routine replaces unsafe characters in $string with their entity
 representation. A second argument can be given to specify which
 characters to consider unsafe (i.e., which to escape). The default set
 of characters to encode are control chars, high-bit chars, and the
-C<< < >>, C<< & >>, C<< > >>, and C<< " >>
+C<< < >>, C<< & >>, C<< > >>, C<< ' >> and C<< " >>
 characters.  But this, for example, would encode I<just> the
 C<< < >>, C<< & >>, C<< > >>, and C<< " >> characters:
 
@@ -53,7 +79,9 @@ C<< < >>, C<< & >>, C<< > >>, and C<< " >> characters:
 
 This routine is exported by default.
 
-=item encode_entities_numeric($string, [$unsafe_chars])
+=item encode_entities_numeric( $string )
+
+=item encode_entities_numeric( $string, $unsafe_chars )
 
 This routine works just like encode_entities, except that the replacement
 entities are always C<&#xI<hexnum>;> and never C<&I<entname>;>.  For
@@ -84,7 +112,7 @@ corresponding entities (and vice versa, respectively).
 
 =head1 COPYRIGHT
 
-Copyright 1995-2003 Gisle Aas. All rights reserved.
+Copyright 1995-2004 Gisle Aas. All rights reserved.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -102,7 +130,7 @@ require Exporter;
 @EXPORT = qw(encode_entities decode_entities _decode_entities);
 @EXPORT_OK = qw(%entity2char %char2entity encode_entities_numeric);
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.26 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.32 $ =~ /(\d+)\.(\d+)/);
 sub Version { $VERSION; }
 
 require HTML::Parser;  # for fast XS implemented decode_entities
@@ -117,106 +145,106 @@ require HTML::Parser;  # for fast XS implemented decode_entities
  apos   => "'",  # single quote
 
  # PUBLIC ISO 8879-1986//ENTITIES Added Latin 1//EN//HTML
- AElig	=> '',  # capital AE diphthong (ligature)
- Aacute	=> '',  # capital A, acute accent
- Acirc	=> '',  # capital A, circumflex accent
- Agrave	=> '',  # capital A, grave accent
- Aring	=> '',  # capital A, ring
- Atilde	=> '',  # capital A, tilde
- Auml	=> '',  # capital A, dieresis or umlaut mark
- Ccedil	=> '',  # capital C, cedilla
- ETH	=> '',  # capital Eth, Icelandic
- Eacute	=> '',  # capital E, acute accent
- Ecirc	=> '',  # capital E, circumflex accent
- Egrave	=> '',  # capital E, grave accent
- Euml	=> '',  # capital E, dieresis or umlaut mark
- Iacute	=> '',  # capital I, acute accent
- Icirc	=> '',  # capital I, circumflex accent
- Igrave	=> '',  # capital I, grave accent
- Iuml	=> '',  # capital I, dieresis or umlaut mark
- Ntilde	=> '',  # capital N, tilde
- Oacute	=> '',  # capital O, acute accent
- Ocirc	=> '',  # capital O, circumflex accent
- Ograve	=> '',  # capital O, grave accent
- Oslash	=> '',  # capital O, slash
- Otilde	=> '',  # capital O, tilde
- Ouml	=> '',  # capital O, dieresis or umlaut mark
- THORN	=> '',  # capital THORN, Icelandic
- Uacute	=> '',  # capital U, acute accent
- Ucirc	=> '',  # capital U, circumflex accent
- Ugrave	=> '',  # capital U, grave accent
- Uuml	=> '',  # capital U, dieresis or umlaut mark
- Yacute	=> '',  # capital Y, acute accent
- aacute	=> '',  # small a, acute accent
- acirc	=> '',  # small a, circumflex accent
- aelig	=> '',  # small ae diphthong (ligature)
- agrave	=> '',  # small a, grave accent
- aring	=> '',  # small a, ring
- atilde	=> '',  # small a, tilde
- auml	=> '',  # small a, dieresis or umlaut mark
- ccedil	=> '',  # small c, cedilla
- eacute	=> '',  # small e, acute accent
- ecirc	=> '',  # small e, circumflex accent
- egrave	=> '',  # small e, grave accent
- eth	=> '',  # small eth, Icelandic
- euml	=> '',  # small e, dieresis or umlaut mark
- iacute	=> '',  # small i, acute accent
- icirc	=> '',  # small i, circumflex accent
- igrave	=> '',  # small i, grave accent
- iuml	=> '',  # small i, dieresis or umlaut mark
- ntilde	=> '',  # small n, tilde
- oacute	=> '',  # small o, acute accent
- ocirc	=> '',  # small o, circumflex accent
- ograve	=> '',  # small o, grave accent
- oslash	=> '',  # small o, slash
- otilde	=> '',  # small o, tilde
- ouml	=> '',  # small o, dieresis or umlaut mark
- szlig	=> '',  # small sharp s, German (sz ligature)
- thorn	=> '',  # small thorn, Icelandic
- uacute	=> '',  # small u, acute accent
- ucirc	=> '',  # small u, circumflex accent
- ugrave	=> '',  # small u, grave accent
- uuml	=> '',  # small u, dieresis or umlaut mark
- yacute	=> '',  # small y, acute accent
- yuml	=> '',  # small y, dieresis or umlaut mark
+ AElig	=> chr(198),  # capital AE diphthong (ligature)
+ Aacute	=> chr(193),  # capital A, acute accent
+ Acirc	=> chr(194),  # capital A, circumflex accent
+ Agrave	=> chr(192),  # capital A, grave accent
+ Aring	=> chr(197),  # capital A, ring
+ Atilde	=> chr(195),  # capital A, tilde
+ Auml	=> chr(196),  # capital A, dieresis or umlaut mark
+ Ccedil	=> chr(199),  # capital C, cedilla
+ ETH	=> chr(208),  # capital Eth, Icelandic
+ Eacute	=> chr(201),  # capital E, acute accent
+ Ecirc	=> chr(202),  # capital E, circumflex accent
+ Egrave	=> chr(200),  # capital E, grave accent
+ Euml	=> chr(203),  # capital E, dieresis or umlaut mark
+ Iacute	=> chr(205),  # capital I, acute accent
+ Icirc	=> chr(206),  # capital I, circumflex accent
+ Igrave	=> chr(204),  # capital I, grave accent
+ Iuml	=> chr(207),  # capital I, dieresis or umlaut mark
+ Ntilde	=> chr(209),  # capital N, tilde
+ Oacute	=> chr(211),  # capital O, acute accent
+ Ocirc	=> chr(212),  # capital O, circumflex accent
+ Ograve	=> chr(210),  # capital O, grave accent
+ Oslash	=> chr(216),  # capital O, slash
+ Otilde	=> chr(213),  # capital O, tilde
+ Ouml	=> chr(214),  # capital O, dieresis or umlaut mark
+ THORN	=> chr(222),  # capital THORN, Icelandic
+ Uacute	=> chr(218),  # capital U, acute accent
+ Ucirc	=> chr(219),  # capital U, circumflex accent
+ Ugrave	=> chr(217),  # capital U, grave accent
+ Uuml	=> chr(220),  # capital U, dieresis or umlaut mark
+ Yacute	=> chr(221),  # capital Y, acute accent
+ aacute	=> chr(225),  # small a, acute accent
+ acirc	=> chr(226),  # small a, circumflex accent
+ aelig	=> chr(230),  # small ae diphthong (ligature)
+ agrave	=> chr(224),  # small a, grave accent
+ aring	=> chr(229),  # small a, ring
+ atilde	=> chr(227),  # small a, tilde
+ auml	=> chr(228),  # small a, dieresis or umlaut mark
+ ccedil	=> chr(231),  # small c, cedilla
+ eacute	=> chr(233),  # small e, acute accent
+ ecirc	=> chr(234),  # small e, circumflex accent
+ egrave	=> chr(232),  # small e, grave accent
+ eth	=> chr(240),  # small eth, Icelandic
+ euml	=> chr(235),  # small e, dieresis or umlaut mark
+ iacute	=> chr(237),  # small i, acute accent
+ icirc	=> chr(238),  # small i, circumflex accent
+ igrave	=> chr(236),  # small i, grave accent
+ iuml	=> chr(239),  # small i, dieresis or umlaut mark
+ ntilde	=> chr(241),  # small n, tilde
+ oacute	=> chr(243),  # small o, acute accent
+ ocirc	=> chr(244),  # small o, circumflex accent
+ ograve	=> chr(242),  # small o, grave accent
+ oslash	=> chr(248),  # small o, slash
+ otilde	=> chr(245),  # small o, tilde
+ ouml	=> chr(246),  # small o, dieresis or umlaut mark
+ szlig	=> chr(223),  # small sharp s, German (sz ligature)
+ thorn	=> chr(254),  # small thorn, Icelandic
+ uacute	=> chr(250),  # small u, acute accent
+ ucirc	=> chr(251),  # small u, circumflex accent
+ ugrave	=> chr(249),  # small u, grave accent
+ uuml	=> chr(252),  # small u, dieresis or umlaut mark
+ yacute	=> chr(253),  # small y, acute accent
+ yuml	=> chr(255),  # small y, dieresis or umlaut mark
 
  # Some extra Latin 1 chars that are listed in the HTML3.2 draft (21-May-96)
- copy   => '',  # copyright sign
- reg    => '',  # registered sign
- nbsp   => "\240", # non breaking space
+ copy   => chr(169),  # copyright sign
+ reg    => chr(174),  # registered sign
+ nbsp   => chr(160),  # non breaking space
 
  # Additional ISO-8859/1 entities listed in rfc1866 (section 14)
- iexcl  => '',
- cent   => '',
- pound  => '',
- curren => '',
- yen    => '',
- brvbar => '',
- sect   => '',
- uml    => '',
- ordf   => '',
- laquo  => '',
-'not'   => '',    # not is a keyword in perl
- shy    => '',
- macr   => '',
- deg    => '',
- plusmn => '',
- sup1   => '',
- sup2   => '',
- sup3   => '',
- acute  => '',
- micro  => '',
- para   => '',
- middot => '',
- cedil  => '',
- ordm   => '',
- raquo  => '',
- frac14 => '',
- frac12 => '',
- frac34 => '',
- iquest => '',
-'times' => '',    # times is a keyword in perl
- divide => '',
+ iexcl  => chr(161),
+ cent   => chr(162),
+ pound  => chr(163),
+ curren => chr(164),
+ yen    => chr(165),
+ brvbar => chr(166),
+ sect   => chr(167),
+ uml    => chr(168),
+ ordf   => chr(170),
+ laquo  => chr(171),
+'not'   => chr(172),    # not is a keyword in perl
+ shy    => chr(173),
+ macr   => chr(175),
+ deg    => chr(176),
+ plusmn => chr(177),
+ sup1   => chr(185),
+ sup2   => chr(178),
+ sup3   => chr(179),
+ acute  => chr(180),
+ micro  => chr(181),
+ para   => chr(182),
+ middot => chr(183),
+ cedil  => chr(184),
+ ordm   => chr(186),
+ raquo  => chr(187),
+ frac14 => chr(188),
+ frac12 => chr(189),
+ frac34 => chr(190),
+ iquest => chr(191),
+'times' => chr(215),    # times is a keyword in perl
+ divide => chr(247),
 
  ( $] > 5.007 ? (
    OElig    => chr(338),
@@ -426,8 +454,8 @@ sub encode_entities
 	}
 	&{$subst{$_[1]}}($$ref);
     } else {
-	# Encode control chars, high bit chars and '<', '&', '>', '"'
-	$$ref =~ s/([^\n\r\t !\#\$%\'-;=?-~])/$char2entity{$1} || num_entity($1)/ge;
+	# Encode control chars, high bit chars and '<', '&', '>', ''' and '"'
+	$$ref =~ s/([^\n\r\t !\#\$%\(-;=?-~])/$char2entity{$1} || num_entity($1)/ge;
     }
     $$ref;
 }
